package com.google.inject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.zeroturnaround.javarebel.ClassEventListener;
import org.zeroturnaround.javarebel.LoggerFactory;

public class GuiceComponentReloader implements ClassEventListener {
	private static final ThreadLocal<InjectorImpl> currentInjector = 
		new ThreadLocal<InjectorImpl>();

	private static Map<InjectorImpl, Map<Class, Object>> singletons = 
		new HashMap<InjectorImpl, Map<Class, Object>>();
	private static List<InjectorImpl> injectors = 
		new ArrayList<InjectorImpl>();

	public int priority() {
	  return PRIORITY_CORE;
	}
	
	public void onClassEvent(int eventType, Class klass) {
		for (InjectorImpl injector : injectors) {
			injector.injectors.remove(klass);

			Map<Class, Object> injectorSingletons = singletons.get(injector);
			if (injectorSingletons == null)
				continue;
			
			Object singleton = injectorSingletons.get(klass);
			if (singleton == null)
				continue;
			
			LoggerFactory.getInstance().echo("JavaRebel-Guice: Reconfiguring singleton '" + klass.getName() + "'.");
			injector.injectMembers(singleton);
		}
	}

	public static void setCurrentInjector(InjectorImpl injector) {
		currentInjector.set(injector);
	}


	public static void registerInjector(InjectorImpl injector) {
		injectors.add(injector);
	}

	public static void registerSingleton(Class key, Object singleton) {
		Map<Class, Object> injectorSingletons = singletons.get(currentInjector.get());
		if (injectorSingletons == null) {
			injectorSingletons = new HashMap<Class, Object>();
			singletons.put(currentInjector.get(), injectorSingletons);
		}
		injectorSingletons.put(key, singleton);
	}
}
